<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

/**
 * Description of logout
 *
 * @author Igor Ratsun
 */
class Logout extends MX_Controller {

    public function index() {
        $this->rt_user->logout();
        redirect(base_url($this->config->item('admin_page')));
    }

}

