<?php

if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Create safe mailto link
 *
 * @author Igor Ratsun
 */
class parser_safe_mailto
{

    public function parse($email = '', $title = '', $attrs = '')
    {
        get_instance()->load->helper('url');
        
        return safe_mailto($email, $title, $attrs);
    }

}
