<?php

if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

/**
 * Date
 *
 * @author Igor Ratsun
 */
class parser_date
{

    public function parse($format = 'd.m.Y', $date = false)
    {
        return empty($date) ? date($format) : date($format, strtotime($date));
    }

}
